@@ -2,7 +2,7 @@ package RPC::Async::Client;
 use strict;
 use warnings;
 
-our $VERSION = '1.02';
+our $VERSION = '1.03';
 
 =head1 NAME
 
@@ -68,8 +68,6 @@ sub new {
         requests => {},
         serial => -1,
         buf => undef,
-        check_request => undef,
-        check_response => undef,
         coderefs => {},
     );
     my ($fh, @urlargs) = url_connect($url, @args); 
@@ -106,17 +104,15 @@ procedures may return in a different order than they were called in.
 Fairly complex data structures may be given as arguments, except for circular
 ones. In particular, subroutine references are allowed.
 
+The call itself is given a uniq id that is returned and can later be used
+with other subs.
+
 =cut
 
 sub call {
     my ($self, $procedure, @args) = @_;
     my $callback = pop @args;
 
-    if ($self->{check_request}
-            and not $self->{check_request}->($procedure, @args)) {
-        croak "Invalid procedure or arguments in call to $procedure\n";
-    }
-
     @args = $self->_encode_args(@args);
 
     my $id = $self->_unique_id;
@@ -124,16 +120,8 @@ sub call {
 
     #print "RPC::Async::Client sending: $id $procedure @args\n";
     $self->{mux}->send($self->{fh}, make_packet([ $id, $procedure, @args ]));
-}
 
-sub check_request {
-    $_[0]->{check_request} = $_[1] if @_ > 1;
-    $_[0]->{check_request};
-}
-
-sub check_response {
-    $_[0]->{check_response} = $_[1] if @_ > 1;
-    $_[0]->{check_response};
+    return $id;
 }
 
 =item disconnect
@@ -197,7 +185,7 @@ sub io {
             die __PACKAGE__ .": server disconnected\n";
         }
 
-        return undef;
+        return;
 
     } else {
         return $event;
@@ -215,16 +203,67 @@ sub dump_requests {
     return Dumper($self->{requests});
 }
 
+=item wait($timeout, [@ids])
+
+Block until we have enough events from the RPC server to matches all the
+id's listed or we get a timeout. If no id's are given then any id will be 
+used. This function can be useful when we want to make sure the server is 
+started in the other end, fx. if we need to connect to it on another socket. 
+
+If all events was received in time, 1 is returned, C<undef> is returned on 
+timeout.
+
+Any unrelated events will be buffered and pushed back on the stack when the
+wait call has finished.
+
+=cut
+
+sub wait {
+    my ($self, $timeout, @ids) = @_;
+    my %match = map { $_ => 1 } @ids;
+    my $count = (int @ids or 1);
+    my $mux = $self->{mux};
+     
+    # Set timeout only on our own fh.
+    $mux->timeout($self->{fh}, $timeout);
+    my @events;
+    while($count and my $event = $mux->mux()) {
+        if ($event->{fh} and $event->{fh} == $self->{fh}) {
+            if ($event->{type} eq "read") {
+                foreach my $id ($self->_handle_read($event->{data})) {
+                    if(exists $match{$id} or @ids == 0) {
+                        $count--;
+                    }
+                }
+
+            } elsif($event->{type} eq 'timeout') {
+                last;
+
+            } elsif ($event->{type} eq "closed") {
+                die __PACKAGE__ .": server disconnected\n";
+            }
+
+        } else {
+            push @events, $event;
+        }
+    }
+
+    $mux->push_event(@events);
+    return $count > 0?undef:1;
+}
+
 sub _handle_read {
     my ($self, $data) = @_;
-    
+    my @ids;
+
     # TODO: Use buffering code in EventMux and remove functions from Util.pm
     append_data(\$self->{buf}, $data);
     while (my $thawed = read_packet(\$self->{buf})) {
         if (ref $thawed eq "ARRAY" and @$thawed >= 1) {
             my ($id, @args) = @$thawed;
             my $callback = delete $self->{requests}{$id};
-            # TODO: test check_response
+            
+            push(@ids, $id);
 
             if (defined $callback) {
                 #print __PACKAGE__, ": callback(@args)\n";
@@ -247,6 +286,8 @@ sub _handle_read {
             warn __PACKAGE__.": Bad data in thawed packet";
         }
     }
+    
+    return @ids;
 }
 
 sub _unique_id {
